'---------------------------------------------------------------------
'  Copyright (C) Microsoft Corporation.  All rights reserved.
' 
'THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
'KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
'IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
'PARTICULAR PURPOSE.
'---------------------------------------------------------------------

Option Explicit On
Option Strict On

Imports System.ComponentModel
Imports System.Reflection
Imports System.Collections

Public Class SortableSearchableList(Of T)
    Inherits bindinglist(Of T)

    Public Sub New()
    End Sub

    Public Overrides Sub EndNew(ByVal itemIndex As Integer)
        If (sortPropertyValue IsNot Nothing And itemIndex = Me.Count - 1) Then
            ApplySortCore(Me.sortPropertyValue, Me.sortDirectionValue)
        End If
        MyBase.EndNew(itemIndex)
    End Sub

    Dim sortDirectionValue As ListSortDirection
    Dim sortPropertyValue As PropertyDescriptor
    Dim isSortedValue As Boolean

    Protected Overrides ReadOnly Property SortPropertyCore() As PropertyDescriptor
        Get
            Return sortPropertyValue
        End Get
    End Property

    Protected Overrides ReadOnly Property SortDirectionCore() As ListSortDirection
        Get
            Return sortDirectionValue
        End Get
    End Property

    Protected Overrides ReadOnly Property IsSortedCore As Boolean
        Get
            Return isSortedValue
        End Get
    End Property

    Protected Overrides ReadOnly Property SupportsSortingCore() As Boolean
        Get
            Return True
        End Get
    End Property

    Public Sub RemoveSort()
        RemoveSortCore()
    End Sub

    Private sortedList As ArrayList
    Private unsortedItems As ArrayList

    Protected Overrides Sub ApplySortCore(ByVal prop As PropertyDescriptor, _
    ByVal direction As ListSortDirection)
        sortedList = New ArrayList

        ' Check to see if the property type we are sorting by implements
        ' the IComparable interface.

        Dim interfaceType As Type = prop.PropertyType.GetInterface("IComparable")

        If interfaceType IsNot Nothing Then

            ' If so, set the SortPropertyValue and SortDirectionValue.
            sortPropertyValue = prop
            sortDirectionValue = direction

            unsortedItems = New ArrayList(Me.Count)

            ' Loop through each item, adding it the the sortedItems ArrayList.
            For Each item As Object In Me.Items
                sortedList.Add(prop.GetValue(item))
                unsortedItems.Add(item)
            Next

            ' Call Sort on the ArrayList.
            sortedList.Sort()
            Dim temp As T

            ' Check the sort direction and then copy the sorted items
            ' back into the list.
            If (direction = ListSortDirection.Descending) Then
                sortedList.Reverse()
            End If

            For i As Integer = 0 To Me.Count - 1
                Dim position As Integer = Find(prop.Name, sortedList(i))
                If position <> i Then
                    temp = Me(i)
                    Me(i) = Me(position)
                    Me(position) = temp
                End If
            Next

            isSortedValue = true

            ' Raise the ListChanged event so bound controls refresh their
            ' values.
            OnListChanged(New ListChangedEventArgs(ListChangedType.Reset, -1))
        Else
            ' If the property type does not implement IComparable, let the user
            ' know.
            Throw New NotSupportedException("Cannot sort by " & prop.Name & ". This" & _
                    prop.PropertyType.ToString() & " does not implement IComparable")
        End If
    End Sub

    Protected Overrides Sub RemoveSortCore()
        Dim position As Integer
        Dim temp As Object

        ' Ensure the list has been sorted.
        If unsortedItems IsNot Nothing Then
            ' Loop through the unsorted items and reorder the
            ' list per the unsorted list.
            Dim i As Integer = 0
            While i < unsortedItems.Count
                position = Me.Find("LastName", _
                        unsortedItems(i).GetType(). _
                        GetProperty("LastName").GetValue(unsortedItems(i), Nothing))
                If position >= 0 And position <> i Then
                    temp = Me(i)
                    Me(i) = Me(position)
                    Me(position) = CType(temp, T)
                    i += 1
                ElseIf position = i Then
                    i += 1
                Else
                    ' If an item in the unsorted list no longer exists, delete it.
                    unsortedItems.RemoveAt(i)
                End If
            End While

            isSortedValue = false

            OnListChanged(New ListChangedEventArgs(ListChangedType.Reset, -1))
        End If
    End Sub

    Protected Overrides ReadOnly Property SupportsSearchingCore() As Boolean
        Get
            Return True
        End Get
    End Property

    Protected Overrides Function FindCore( _
    ByVal prop As System.ComponentModel.PropertyDescriptor, _
    ByVal key As Object) As Integer
        ' Get the property info for the specified property.
        Dim propInfo As PropertyInfo = GetType(T).GetProperty(prop.Name)
        Dim item As T
        If key IsNot Nothing Then
            ' Loop through the the items to see if the key
            ' value matches the property value.
            For i As Integer = 0 To Count - 1
                item = CType(Items(i), T)
                If propInfo.GetValue(item, Nothing).Equals(key) Then
                    Return i
                End If
            Next
            Return -1
        End If
    End Function

    Public Function Find(ByVal propertyName As String, ByVal key As Object) As Integer
        ' Check the properties for a property with the specified name.
        Dim properties As PropertyDescriptorCollection = _
            TypeDescriptor.GetProperties(GetType(T))
        Dim prop As PropertyDescriptor = properties.Find(propertyName, True)

        ' If there is not a match, return -1 otherwise pass search to
        ' FindCore method.
        If prop Is Nothing Then
            Return -1
        Else
            Return FindCore(prop, key)
        End If
    End Function

End Class
